package com.eco.plugin.xx.zjgintegrate.sso.pc.filter;

import com.eco.plugin.xx.zjgintegrate.config.PCSSSOConfig;
import com.eco.plugin.xx.zjgintegrate.utils.FRUtils;
import com.eco.plugin.xx.zjgintegrate.utils.HttpUtils;
import com.eco.plugin.xx.zjgintegrate.utils.Utils;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.fun.Authorize;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;

@EnableMetrics
@Authorize(callSignKey = "com.eco.plugin.xx.zjgintegrate")
public class SSOFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "fqlssoFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain chain ){

        if(PluginContexts.currentContext().isAvailable()){
            PCSSSOConfig psc = PCSSSOConfig.getInstance();

            //是否放行
            boolean release = isRelease(req);
            if(release){
                FRUtils.FRLogInfo("");
                release(req,res,chain);
                return;
            }

            String code = req.getParameter("code");

            if(Utils.isNullStr(code)){
                //跳转认证中心
                redirect(req,res,psc);
                return;
            }

            //获取token
            String token = getToken(code,psc);

            //获取userInfo
            String username = getUsername(token,psc);
            String redirect = Utils.getRedirectUrl(req,"code");
            //登录
            FRUtils.login(req,res,username,redirect);
        }

        release(req,res,chain);
    }

    private boolean isRelease(HttpServletRequest req) {
        String url = FRUtils.getAllUrl(req);
        String origin = req.getParameter("origin");
        String code = req.getParameter("code");
        if((url.contains("/login") && Utils.isNotNullStr(origin)) || Utils.isNotNullStr(code)){
            return false;
        }

        return true;
    }

    //跳转认证中心
    private void redirect(HttpServletRequest req,HttpServletResponse res, PCSSSOConfig psc) {
        String redirect = "";
        try {
            redirect = FRUtils.getOriginUrl(req);
            redirect = URLEncoder.encode(redirect);
        } catch (Exception e) {
            FRUtils.FRLogInfo("获取redirect异常:"+e.getMessage());
        }
        String authurl = psc.getAuthurl()+"?redirect_uri="+redirect+"&state=1&client_id="+psc.getClientid()+"&response_type=code";

        try {
            res.sendRedirect(authurl);
        } catch (IOException e) {
            FRUtils.FRLogInfo("跳转认证中心异常:"+e.getMessage());
        }
    }

    //获取token
    private String getToken(String code,PCSSSOConfig psc) {
        String tokenurl = psc.getTokenurl()+"?client_id="+psc.getClientid()+"&grant_type=authorization_code&code="+code+"&client_secret="+psc.getSecret();

        String result = HttpUtils.HttpPostJson(tokenurl,"",null);

        if(Utils.isNullStr(result)){
            return "";
        }

        JSONObject json = new JSONObject(result);

        if(json == null ){
            return "";
        }

        String token = json.getString("access_token");
        return token;
    }

    //获取用户名
    private String getUsername(String openId,PCSSSOConfig psc) {
        String tokenurl = psc.getUserurl()+"?access_token="+openId+"&client_id="+psc.getClientid();

        String result = HttpUtils.httpGet(tokenurl,null,null);

        if(Utils.isNullStr(result)){
            return "";
        }

        JSONObject json = new JSONObject(result);

        if(json == null ){
            return "";
        }

        String username = json.getString("loginName");
        return username;
    }

    //放行拦截器
    private void release(HttpServletRequest req, HttpServletResponse res, FilterChain chain) {
        try{
            chain.doFilter(req,res);
        }catch (Exception e){
            FRUtils.FRLogInfo("拦截失败");
        }
    }
}

