package com.eco.plugin.xx.zjgintegrate.sync.controller;

import com.eco.plugin.xx.zjgintegrate.config.MobileSSSOConfig;
import com.eco.plugin.xx.zjgintegrate.config.PluginSimpleConfig;
import com.eco.plugin.xx.zjgintegrate.kit.DepartmentServiceKit;
import com.eco.plugin.xx.zjgintegrate.kit.UserServiceKit;
import com.eco.plugin.xx.zjgintegrate.utils.*;
import com.fr.decision.authority.data.Department;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.GetMapping;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Controller
@LoginStatusChecker(required = false)
@FunctionRecorder
public class ControllerSelf {

    /**
     * 帆软用户组织字段信息
     * @param req
     * @param res
     */
    @PostMapping(value = "/SchemaService")
    @ResponseBody
    public void SchemaService(HttpServletRequest req,HttpServletResponse res){
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param,res);

        if(auth){
            JSONObject result = getResultJson(param);

            //用户字段
            JSONArray userField = new JSONArray();
            String id = "{\"multivalued\":false,\"name\":\"uid\",\"required\":true,\"type\":\"String\"}";
            String username = "{\"multivalued\":false,\"name\":\"username\",\"required\":true,\"type\":\"String\"}";
            String realname = "{\"multivalued\":false,\"name\":\"realname\",\"required\":true,\"type\":\"String\"}";
            String orgId = "{\"multivalued\":false,\"name\":\"orgId\",\"required\":false,\"type\":\"String\"}";
            String mobile = "{\"multivalued\":false,\"name\":\"mobile\",\"required\":false,\"type\":\"String\"}";
            String email = "{\"multivalued\":false,\"name\":\"email\",\"required\":false,\"type\":\"String\"}";
            userField.add(new JSONObject(id)).add(new JSONObject(username)).add(new JSONObject(realname))
                    .add(new JSONObject(orgId)).add(new JSONObject(mobile)).add(new JSONObject(email));
            result.put("account",userField);

            //部门字段
            JSONArray deptField = new JSONArray();
            String orgid = "{\"multivalued\":false,\"name\":\"orgId\",\"required\":true,\"type\":\"String\"}";
            String orgName = "{\"multivalued\":false,\"name\":\"orgName\",\"required\":true,\"type\":\"String\"}";
            String pid = "{\"multivalued\":false,\"name\":\"pid\",\"required\":false,\"type\":\"String\"}";
            deptField.add(new JSONObject(orgid)).add(new JSONObject(orgName)).add(new JSONObject(pid));
            result.put("organization",deptField);

            ResponseUtils.response(res,result);
        }
    }


    /**
     * 创建组织
     * @param req
     * @param res
     */
    @PostMapping(value = "/OrgCreateService")
    @ResponseBody
    public void OrgCreateService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("orgId");
            String orgName = param.getString("orgName");
            String pid = param.getString("pid");

            if(Utils.isNullStr(id)){
                id = UUID.randomUUID().toString();
            }

            try {
                DepartmentServiceKit.getInstance().addDepartment(id,pid,orgName);
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","创建失败");
                FRUtils.FRLogInfo("OrgCreateService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("orgId",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 创建组织
     * @param req
     * @param res
     */
    @PostMapping(value = "/OrgUpdateService")
    @ResponseBody
    public void OrgUpdateService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("orgId");
            String orgName = param.getString("orgName");
            String pid = param.getString("pid");

            try {
                Department department = DepartmentServiceKit.getInstance().getByid(id);

                if(Utils.isNullStr(orgName)){
                    orgName = department.getName();
                }

                if(pid == null){
                    pid = department.getParentId();
                }

                DepartmentServiceKit.getInstance().editDepartment(id,orgName,pid);
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","修改失败");
                FRUtils.FRLogInfo("OrgUpdateService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("orgId",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 创建组织
     * @param req
     * @param res
     */
    @PostMapping(value = "/OrgDeleteService")
    @ResponseBody
    public void OrgDeleteService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("orgId");

            try {
                DepartmentServiceKit.getInstance().deleteDepartment(id);
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","删除失败");
                FRUtils.FRLogInfo("OrgDeleteService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("orgId",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 创建用户
     * @param req
     * @param res
     */
    @PostMapping(value = "/UserCreateService")
    @ResponseBody
    public void UserCreateService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("uid");
            String username = param.getString("username");
            String realname = param.getString("realname");
            String orgId = param.getString("orgId");
            String mobile = param.getString("mobile");
            String email = param.getString("email");
            String psd = "123456";
            boolean status = param.getInt("status") == 0;

            if(Utils.isNullStr(id)){
                id = username;
            }

            try {
                UserBean userBean = new UserBean();
                userBean.setUsername(username);
                userBean.setRealName(realname);
                userBean.setMobile(mobile);
                userBean.setEmail(email);
                userBean.setPassword(psd);
                userBean.setId(id);
                UserServiceKit.getInstance().addUser(userBean);
                User user = FRUserUtils.getUserByUserName(username);
                FRUserUtils.forbidUser(user.getId(),status);
                UserUpdateBean userUpdateBean = new UserUpdateBean();
                userUpdateBean.setAddUserIds(new String[]{user.getId()});
                FRUserUtils.updateDepartmentPostUsers(orgId,"",userUpdateBean);
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","添加失败");
                FRUtils.FRLogInfo("UserCreateService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("uid",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 创建用户
     * @param req
     * @param res
     */
    @PostMapping(value = "/UserUpdateService")
    @ResponseBody
    public void UserUpdateService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("uid");
            String username = param.getString("username");
            String realname = param.getString("realname");
            String orgId = param.getString("orgId");
            String mobile = param.getString("mobile");
            String email = param.getString("email");
            String statusStr = param.getString("status");
            boolean status = param.getInt("status") == 0;

            try {
                User user = FRUserUtils.getUserByUserName(FRUserUtils.getUser(id).getUsername());

                if(Utils.isNullStr(realname)){
                    realname = user.getRealName();
                }

                if(Utils.isNullStr(mobile)){
                    mobile = user.getMobile();
                }

                if(Utils.isNullStr(email)){
                    email = user.getEmail();
                }

                UserBean userBean = new UserBean();
                userBean.setId(user.getId());
                userBean.setUsername(user.getUserName());
                userBean.setRealName(realname);
                userBean.setMobile(mobile);
                userBean.setEmail(email);
                FRUserUtils.updateUser(userBean);
                if(!Utils.isNullStr(statusStr)){
                    FRUserUtils.forbidUser(user.getId(),status);
                }

                if(!Utils.isNullStr(orgId)){
                    UserUpdateBean userUpdateBean = new UserUpdateBean();
                    userUpdateBean.setAddUserIds(new String[]{user.getId()});
                    FRUserUtils.updateDepartmentPostUsers(orgId,"",userUpdateBean);
                }
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","修改失败");
                FRUtils.FRLogInfo("UserUpdateService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("uid",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 创建用户
     * @param req
     * @param res
     */
    @PostMapping(value = "/UserDeleteService")
    @ResponseBody
    public void UserDeleteService(HttpServletRequest req,HttpServletResponse res) {
        JSONObject param = Utils.getRequestBody(req);
        boolean auth = auth(param, res);

        if (auth) {
            JSONObject result = getResultJson(param);
            String id = param.getString("uid");

            try {
                User user = FRUserUtils.getUserByUserName(FRUserUtils.getUser(id).getUsername());
                FRUserUtils.deleteUser(user);
            } catch (Exception e) {
                result.put("resultCode","-1");
                result.put("message","删除失败");
                FRUtils.FRLogInfo("UserDeleteService exception:"+e.getMessage());

                ResponseUtils.response(res,result);
                return ;
            }

            result.put("uid",id);
            ResponseUtils.response(res,result);
        }
    }

    /**
     * 获取appkey
     * @param req
     * @param res
     */
    @PostMapping(value = "/getAppkey")
    @ResponseBody
    public void getAppkey(HttpServletRequest req,HttpServletResponse res) {
        JSONObject json = new JSONObject();
        json.put("appKey",MobileSSSOConfig.getInstance().getAppkey());
        ResponseUtils.response(res,json);
    }

    /**
     * 单点登陆
     * @param req
     * @param res
     */
    @GetMapping(value = "/ssoLogin")
    @ResponseBody
    public void ssoLogin(HttpServletRequest req,HttpServletResponse res) {
        String ticket = req.getParameter("ticket");
        String redirect = req.getParameter("redirect");
        MobileSSSOConfig psc = MobileSSSOConfig.getInstance();
        String url = psc.getUserurl();
        JSONObject param = new JSONObject();
        param.put("device",psc.getAppkey());
        param.put("encryptedString",ticket);

        Map<String,String> header = new HashMap<String,String>();
        header.put("operator","hw");

        FRUtils.FRLogInfo("header:"+header.toString());
        FRUtils.FRLogInfo("param:"+param.toString());

        String resultStr = HttpUtils.HttpPostJson(url,param.toString(),header);

        if(Utils.isNullStr(resultStr)){
            ResponseUtils.failedResponse(res,"获取用户信息失败!");
            return ;
        }

        JSONObject resultJson = new JSONObject(resultStr);
        String code = resultJson.getString("code");

        if(!"info.common.success".equals(code)){
            ResponseUtils.failedResponse(res,"获取用户信息失败!");
            return ;
        }

        String username = resultJson.getJSONObject("body").getString("username");

        FRUtils.login(req,res,username,Utils.encodeCH(redirect));
    }

    /**
     * 获取返回值json
     * @param param
     * @return
     */
    private static JSONObject getResultJson(JSONObject param){
        String iamRequestId = param.getString("iamRequestId");
        JSONObject result = new JSONObject();
        result.put("iamRequestId",iamRequestId);
        result.put("resultCode","0");
        result.put("message","成功");

        return result;
    }

    /**
     * 鉴权
     * @param param
     * @param res
     * @return
     */
    private static boolean auth(JSONObject param, HttpServletResponse res){
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        String iamRemoteUser = param.getString("iamRemoteUser");
        String iamRemotePwd = param.getString("iamRemotePwd");

        boolean success = iamRemoteUser.equals(psc.getUsername()) && iamRemotePwd.equals(psc.getPsd());

        if(!success){
            JSONObject result = new JSONObject();
            result.put("resultCode","302");
            result.put("message","鉴权失败!");
            ResponseUtils.response(res,result);
            return false;
        }

        return true;
    }
}
